@@ -7,8 +7,6 @@ from sakkara.model.base import ModelComponent
 from sakkara.model.math_op import MathOpBase
 from sakkara.relation.groupset import GroupSet
 
-from sakkara.relation.representation import TensorRepresentation
-
 S = TypeVar('S', bound=Any)
 T = TypeVar('T', bound=ModelComponent)
 
@@ -59,19 +57,6 @@ class Composable(MathOpBase, ABC, Generic[S, T]):
             if component.variable is None:
                 component.build(groupset)
 
-    def build_representation(self, groupset: GroupSet):
-        self.representation = TensorRepresentation(groupset['global'])
-        self.base_representation = TensorRepresentation(groupset['global'])
-        self.components_representation = TensorRepresentation(groupset['global'])
-
-        for g in self.group:
-            self.base_representation.add_group(groupset[g])
-
-        for component in self.subcomponents.values():
-            self.components_representation = self.components_representation.merge(component.representation)
-
-        self.representation = self.base_representation.merge(self.components_representation)
-
     def clear(self):
         self.variable = None
         self.representation = None
